@@ -1,4 +1,4 @@
-#  Copyright 2017-2021 Reveal Energy Services, Inc 
+#  Copyright (c) 2017-2022 Reveal Energy Services, Inc 
 #
 #  Licensed under the Apache License, Version 2.0 (the "License"); 
 #  you may not use this file except in compliance with the License. 
@@ -46,7 +46,7 @@ def _trajectory_array_in_unit(tgt_unit, raw_array):
     return result
 
 
-class NativeTrajectoryAdapter(dna.DotNetAdapter):
+class NativeTrajectoryAdapterIdentified(dna.IdentifiedDotNetAdapter):
     def __init__(self, net_trajectory: IWellTrajectory):
         """
         Construct an instance adapting a .NET `IWellTrajectory`.
